@@ -22,7 +22,7 @@ class Constants:
     # Constants for producer
     TALOS_SINGLE_MESSAGE_BYTES_MINIMAL = 1
     TALOS_SINGLE_MESSAGE_BYTES_MAXIMAL = 10 * 1024 * 1024
-    TALOS_MESSAGE_BLOCK_BYTES_MAXIMAL = 20 * 1024 * 1024
+    TALOS_MESSAGE_BLOCK_BYTES_MAXIMAL = 40 * 1024 * 1024
 
     TALOS_PARTITION_KEY_LENGTH_MINIMAL = 1
     TALOS_PARTITION_KEY_LENGTH_MAXIMAL = 256
@@ -33,3 +33,20 @@ class Constants:
     TALOS_CLOUD_TEAM_PREFIX = "CI"
     TALOS_CLOUD_AK_PREFIX = "AK"
     TALOS_GALAXY_AK_PREFIX = "EAK"
+
+    # Constants for consumer metrics
+    FETCH_MESSAGE_TIMES = "fetchMessage.60sRate"
+    FETCH_MESSAGE_FAILED_TIMES = "fetchMessageFailed.60sRate"
+    FETCH_MESSAGE_TIME = "fetchMessageTime.gauge"
+    MAX_FETCH_MESSAGE_TIME = "fetchMessageTime.max"
+    MIN_FETCH_MESSAGE_TIME = "fetchMessageTime.min"
+    PROCESS_MESSAGE_TIME = "processMessageTime.gauge"
+    MAX_PROCESS_MESSAGE_TIME = "processMessageTime.max"
+    MIN_PROCESS_MESSAGE_TIME = "processMessageTime.min"
+
+    # Constants for producer metrics
+    PUT_MESSAGE_TIMES = "putMessage.60sRate"
+    PUT_MESSAGE_FAILED_TIMES = "putMessageFailed.60sRate"
+    PUT_MESSAGE_TIME = "putMessageTime.gauge"
+    MAX_PUT_MESSAGE_TIME = "putMessageTime.max"
+    MIN_PUT_MESSAGE_TIME = "putMessageTime.min"
