@@ -11,7 +11,10 @@ from talos.producer.SimpleProducer import SimpleProducer
 from talos.utils.Utils import synchronized
 from talos.utils import Utils
 from talos.producer.PartitionMessageQueue import PartitionMessageQueue
+from talos.producer.TalosProducerConfig import TalosProducerConfig
+from talos.client.Constants import Constants
 from concurrent.futures import ThreadPoolExecutor
+import json
 import logging
 import threading
 import traceback
@@ -22,6 +25,79 @@ class TASK_STATE:
     CLOSE = 1
 
 
+class ProducerMetrics(object):
+    falconEndpoint = str
+    putMsgDuration = int
+    maxPutMsgDuration = int
+    minPutMsgDuration = int
+    putMsgTimes = int
+    putMsgFailedTimes = int
+    producerMetricsMap = dict
+    talosProducerConfig = TalosProducerConfig
+    topicAndPartition = TopicAndPartition
+
+    def __init__(self, talosProducerConfig=None, topicAndPartition=None):
+        self.init_metrics()
+        self.talosProducerConfig = talosProducerConfig
+        self.topicAndPartition = topicAndPartition
+        self.falconEndpoint = self.talosProducerConfig.get_producer_metric_falcon_endpoint() \
+                              + self.topicAndPartition.topicName
+        self.producerMetricsMap = dict()
+
+    def init_metrics(self):
+        self.putMsgDuration = 0
+        self.maxPutMsgDuration = 0
+        self.minPutMsgDuration = 0
+        self.putMsgTimes = 0
+        self.putMsgFailedTimes = 0
+
+    def mark_put_msg_duration(self, putMsgDuration=None):
+        if putMsgDuration > self.maxPutMsgDuration:
+            self.maxPutMsgDuration = putMsgDuration
+
+        if self.minPutMsgDuration == 0 or putMsgDuration < self.minPutMsgDuration:
+            self.minPutMsgDuration = putMsgDuration
+
+        self.putMsgDuration = putMsgDuration
+        self.putMsgTimes += 1
+
+    def mark_put_msg_failed_times(self):
+        self.putMsgFailedTimes += 1
+
+    def to_json_data(self):
+        self.update_metrics_map()
+        jsonArray = list()
+        for key, value in self.producerMetricsMap.items():
+            jsonDict = self.get_basic_data()
+            jsonDict["metric"] = key
+            jsonDict["value"] = value
+            jsonArray.append(jsonDict)
+        self.init_metrics()
+        return jsonArray
+
+    def update_metrics_map(self):
+        self.producerMetricsMap[Constants.PUT_MESSAGE_TIME] = self.putMsgDuration
+        self.producerMetricsMap[Constants.MAX_PUT_MESSAGE_TIME] = self.maxPutMsgDuration
+        self.producerMetricsMap[Constants.MIN_PUT_MESSAGE_TIME] = self.minPutMsgDuration
+        self.producerMetricsMap[Constants.PUT_MESSAGE_TIMES] = self.putMsgTimes / 60.0
+        self.producerMetricsMap[Constants.PUT_MESSAGE_FAILED_TIMES] = self.putMsgFailedTimes/ 60.0
+
+    def get_basic_data(self):
+        tag = "clusterName=" + self.talosProducerConfig.get_cluster_name()
+        tag += ",topicName=" + self.topicAndPartition.topicName
+        tag += ",partitionId=" + str(self.topicAndPartition.partitionId)
+        tag += ",ip=" + self.talosProducerConfig.get_client_ip()
+        tag += ",type=" + self.talosProducerConfig.get_alert_type()
+
+        basicData = dict()
+        basicData["endpoint"] = self.falconEndpoint
+        basicData["timestamp"] = Utils.current_time_mills() / 1000
+        basicData["step"] = self.talosProducerConfig.get_metric_falcon_step() / 1000
+        basicData["counterType"] = "GAUGE"
+        basicData["tags"] = tag
+        return basicData
+
+
 class MessageWriter(threading.Thread):
 
     def __init__(self, partitionSender=None, globalLock=None):
@@ -56,12 +132,10 @@ class MessageWriter(threading.Thread):
 
                 self.partitionSender.put_message(messageList, self.simpleProducer)
             except Exception as e:
-                if isinstance(e, GalaxyTalosException):
-                    self.partitionSender.logger.error(e.errMsg)
                 self.partitionSender.logger.error(
                     "PutMessageTask for topicAndPartition: " +
                     str(self.partitionSender.topicAndPartition)
-                    + " failed" + str(traceback.format_exc()))
+                    + " failed" + e.message)
             finally:
                 # notify to wake up producer's global lock
                 with self.synchronizedLock:
@@ -111,11 +185,13 @@ class PartitionSender:
                                                    partitionId)
         self.partitionMessageQueue = PartitionMessageQueue(talosProducerConfig,
                                                            partitionId, producer)
+        self.producerMetrics = ProducerMetrics(
+            talosProducerConfig=self.talosProducerConfig,
+            topicAndPartition=self.topicAndPartition)
         self.singleExecutor = ThreadPoolExecutor(
             max_workers=1, thread_name_prefix="talos-producer-" +
                                               self.topicAndPartition.topicName
                                               + " " + str(self.topicAndPartition.partitionId))
-
         self.executor = MessageWriter(self, self.globalLock)
         self.executor.start()
 
@@ -134,17 +210,24 @@ class PartitionSender:
                             " reconstruct the TalosProducer again"
                 raise e
             else:
+                startPutMsgTime = Utils.current_time_mills()
                 simpleProducer.do_put(messageList)
+                self.producerMetrics.mark_put_msg_duration(Utils.current_time_mills()
+                                                           - startPutMsgTime)
                 # putMessage success callback
                 userMessageResult.set_successful(True)
+                # self.messageCallbackExecutors = threading.Thread(
+                #     target=self.message_callback_task, args=(userMessageResult, ))
                 self.messageCallbackExecutors = MessageCallbackTask(userMessageResult,
                                                                     self.userMessageCallback)
                 self.messageCallbackExecutors.start()
                 if self.logger.isEnabledFor(logging.DEBUG):
-                    self.logger.debug("put " + messageList.size() +
-                                      " message success for partition: "
+                    self.logger.debug("put " + str(len(messageList))
+                                      + " message success for partition: "
                                       + str(self.partitionId))
         except Exception as e:
+            self.producerMetrics.mark_put_msg_failed_times()
+
             # 1. backoff sleep and retry according to configuration
             # 2. clear partition buffer when long time failed and
             # queue size greater than threshold
@@ -157,6 +240,9 @@ class PartitionSender:
             # execute putMessage failed callback when all retry failed
             userMessageResult.set_successful(False)
             userMessageResult.set_cause(e)
+            # self.messageCallbackExecutors = threading.Thread(
+            #     target=self.message_callback_task,
+            #     args=(self, userMessageResult))
             self.messageCallbackExecutors = MessageCallbackTask(userMessageResult,
                                                                 self.userMessageCallback)
             self.messageCallbackExecutors.start()
@@ -165,11 +251,8 @@ class PartitionSender:
                     self.logger.error(message.sequenceNumber + ": " + str(message.message))
 
             if not Utils.is_partition_not_serving(e):
-                if isinstance(e, GalaxyTalosException):
-                    self.logger.error(e.errMsg)
                 self.logger.error("Failed to put " + str(len(userMessageResult.get_message_list())) +
-                                  " messages for partition: " + str(self.partitionId)
-                                  + str(traceback.format_exc()) )
+                                  " messages for partition: " + str(self.partitionId) + e.message)
 
     def retry_put_message(self, e=None, userMessageResult=None, simpleProducer=None):
         if self.talosProducerConfig.get_max_retry() <= 0:
@@ -182,20 +265,27 @@ class PartitionSender:
             if self.logger.isEnabledFor(logging.DEBUG):
                 self.logger.warn("putMessage failed " + str(retry) +
                                  " times for partition: " + str(self.partitionId) +
-                                 ", sleep " + pauseTime +
+                                 ", sleep " + str(pauseTime) +
                                  "ms for avoid infinitely retry." + e.message)
             # 1. sleep
             Utils.sleep_pause_time(pauseTime)
 
             # 2. retry putMessage, just return if retry success
             try:
+                startPutMsgTime = Utils.current_time_mills()
                 simpleProducer.do_put(userMessageResult.get_message_list())
+                self.producerMetrics.mark_put_msg_duration(Utils.current_time_mills()
+                                                           - startPutMsgTime)
                 userMessageResult.set_successful(True)
+                # self.messageCallbackExecutors = threading.Thread(
+                #     target=self.message_callback_task,
+                #     args=(self, userMessageResult))
                 self.messageCallbackExecutors = MessageCallbackTask(userMessageResult,
                                                                     self.userMessageCallback)
                 self.messageCallbackExecutors.start()
                 return True
             except Exception as ex:
+                self.producerMetrics.mark_put_msg_failed_times()
                 if isinstance(ex, GalaxyTalosException):
                     self.logger.error(ex.errMsg)
                 self.logger.error("retry put message failed. " + str(traceback.format_exc()))
@@ -215,6 +305,8 @@ class PartitionSender:
             self.topicAndPartition.topicName(), self.partitionId)
         bufferedUserMsgResult.set_successful(False)
         bufferedUserMsgResult.set_cause(e)
+        # self.messageCallbackExecutors = threading.Thread(
+        #     target=self.message_callback_task, args=(self, bufferedUserMsgResult))
         self.messageCallbackExecutors = MessageCallbackTask(bufferedUserMsgResult,
                                                             self.userMessageCallback)
         self.messageCallbackExecutors.start()
@@ -235,6 +327,9 @@ class PartitionSender:
                                            " do callback when shutdown this partitionSender."
             galaxyTalosException.errMsg = str(self.topicAndPartition) + "'s buffer not empty"
             userMessageResult.set_cause(galaxyTalosException)
+            # self.messageCallbackExecutors = threading.Thread(
+            #     target=self.message_callback_task,
+            #     args=(self, userMessageResult))
             self.messageCallbackExecutors = MessageCallbackTask(userMessageResult,
                                                                 self.userMessageCallback)
             self.messageCallbackExecutors.start()
@@ -243,6 +338,7 @@ class PartitionSender:
         # notify PartitionMessageQueue::getMessageList return;
         self.add_message([])
         self.update_state(TASK_STATE.CLOSE)
+#        self.singleExecutor.shutdown(True)
         self.executor.join()
         self.logger.info("PartitionSender for partition: "
                          + str(self.topicAndPartition.partitionId) + " finish stop")
@@ -285,3 +381,8 @@ class PartitionSender:
         else:
             return True
 
+    def get_producer_metrics(self):
+        return self.producerMetrics
+
+    def get_falcon_data(self):
+        return self.get_producer_metrics().to_json_data()
