@@ -92,7 +92,7 @@ class SimpleProducer(object):
             Utils.update_message(message, MessageType.BINARY)
 
         # check data validity
-        Utils.check_message_list_validity(msgList)
+        Utils.check_message_list_validity(msgList, self.producerConfig)
 
         self.do_put(msgList)
 
