@@ -69,34 +69,34 @@ def update_message(message=None, messageType=None):
         message.messageType = messageType
 
 
-def check_message_list_validity(msgList=None):
+def check_message_list_validity(msgList=None, producerConfig=None):
     totalSize = 0
     for message in msgList:
-        check_message_validity(message)
+        check_message_validity(message, producerConfig)
         totalSize += len(message.message)
 
     # is too large, and not just throw exception;
-    if totalSize > Constants.TALOS_SINGLE_MESSAGE_BYTES_MAXIMAL * 2:
+    if totalSize > producerConfig.get_max_single_msg_bytes() * 2:
         raise InvalidArgumentError("Total Messages byte must less than " +
-                                   str(Constants.TALOS_SINGLE_MESSAGE_BYTES_MAXIMAL * 2))
+                                   str(producerConfig.get_max_single_msg_bytes() * 2))
 
 
-def check_message_validity(message=None):
-    check_message_len_validity(message)
+def check_message_validity(message=None, producerConfig=None):
+    check_message_len_validity(message, producerConfig)
     check_message_sequence_number_validity(message)
     check_message_type_validity(message)
 
 
-def check_message_len_validity(message=None):
+def check_message_len_validity(message=None, producerConfig=None):
     if not message.message:
         raise InvalidArgumentError("Field \"message\" must be set")
 
     data = message.message
-    if len(data) > Constants.TALOS_SINGLE_MESSAGE_BYTES_MAXIMAL\
-            or len(data) < Constants.TALOS_SINGLE_MESSAGE_BYTES_MINIMAL:
+    max_bytes = producerConfig.get_max_single_msg_bytes() \
+        if producerConfig != None else Constants.TALOS_SINGLE_MESSAGE_BYTES_MAXIMAL
+    if len(data) > max_bytes or len(data) < Constants.TALOS_SINGLE_MESSAGE_BYTES_MINIMAL:
         raise InvalidArgumentError("Data must be less than or equal to " +
-                                   str(Constants.TALOS_SINGLE_MESSAGE_BYTES_MAXIMAL) +
-                                   " bytes, got bytes: " + str(len(data)))
+                                   str(max_bytes) + " bytes, got bytes: " + str(len(data)))
 
 
 def check_message_sequence_number_validity(message=None):
